/*
// File:		ImapMessageData.cs
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2008 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

using System;
using System.Collections.Generic;
using System.Text;
using Emai.Mime;
using Emai.Imap;
using Emai;

namespace Imap
{
    class ImapMessageData
    {
        private Message mMessage;
        private Boolean mFullMessageData;
        private UInt32 mFlags;
        private UInt32 mUid;

        public ImapMessageData()
        {
            this.mMessage = null;
            this.mFullMessageData = false;
            this.mFlags = 0;
            this.mUid = 0;
        }

        public ImapMessageData(UInt32 inUID)
        {
            this.mMessage = null;
            this.mFullMessageData = false;
            this.mFlags = 0;
            this.mUid = inUID;
        }

        public Boolean HasFullMessageData
        {
            get
            {
                return this.mFullMessageData;
            }

            set
            {
                this.mFullMessageData = value;
            }
        }

        public Message Message
        {
            get
            {
                return this.mMessage;
            }

            set
            {
                this.mMessage = value;
            }
        }

        public UInt32 Flags
        {
            get
            {
                return this.mFlags;
            }

            set
            {
                this.mFlags = value;
            }
        }

        public UInt32 Uid
        {
            get
            {
                return this.mUid;
            }

            set
            {
                this.mUid = value;
            }
        }

        public FetchResponse Data
        {
            set
            {
                if (value.MessageFlags.StandardFlags != 0)
                    mFlags = value.MessageFlags.StandardFlags;

                if (value.Uid != 0)
                    mUid = value.Uid;

                if (value.Rfc822 != null)
                {
                    this.mMessage = new Message(value.Rfc822);
                    this.mFullMessageData = true;
                }
                else
                {
                    if (!HasFullMessageData)
                    {
                        if (value.Rfc822Header != null)
                        {
                            this.mMessage = new Message(value.Rfc822Header);
                        }
                        if (value.Rfc822Text != null)
                        {
                            if (this.mMessage != null)
                            {
                                Byte[] data = new Byte[1];
                                UInt32 headerSize = 0;
                                UInt32 dataSize = 0;
                                mMessage.GetData(ref data, ref headerSize, ref dataSize, (UInt32)EnumNullOptions.NullOptions);
                                Byte[] body = new Byte[headerSize + value.Rfc822Text.Length];
                                Array.Copy(data, body, headerSize);
                                Array.Copy(value.Rfc822Text, 0, body, headerSize, value.Rfc822Text.Length);
                                mMessage = new Message(body, (UInt32)EnumNullOptions.NullOptions);
                                this.mFullMessageData = true;
                            }
                            else
                                Console.Out.WriteLine("WARNING: Received text, but do not have header! Do nothing.");
                        }
                    }
                }
            }
        }
    }
}
