/*
// File:		ImapResponseHandler.cs
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2008 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Diagnostics;
using Emai.Imap;
using Emai.Mime;

namespace Imap
{
    using LsubResponse = ListResponse;
    using ExistsResponse = NumberResponse;
    using RecentResponse = NumberResponse;
    using ExpungeResponse = NumberResponse;
    class ImapResponseHandler
    {
        private ArrayList mFolders;
        private ArrayList mSubscribedFolders;
        private ImapSession mSession;
        public ImapResponseHandler(ref ImapSession inSession)
        {
            this.mFolders = new ArrayList();
            this.mSubscribedFolders = new ArrayList();
            this.mSession = inSession;
            this.mSession.ProcessStatus += new Response_ProcessStatusHandler(OnProcessStatus);
            this.mSession.ProcessCapability += new Response_ProcessCapabilityHandler(OnProcessCapability);
            this.mSession.ProcessList += new Response_ProcessListHandler(OnProcessList);
            this.mSession.ProcessLsub += new Response_ProcessLsubHandler(OnProcessLsub);
            this.mSession.ProcessMailboxStatus += new Response_ProcessMailboxStatusHandler(OnProcessMailboxStatus);
            this.mSession.ProcessSearch += new Response_ProcessSearchHandler(OnProcessSearch);
            this.mSession.ProcessFlags += new Response_ProcessFlagsHandler(OnProcessFlags);
            this.mSession.ProcessExists += new Response_ProcessExistsHandler(OnProcessExists);
            this.mSession.ProcessRecent += new Response_ProcessRecentHandler(OnProcessRecent);
            this.mSession.ProcessExpunge += new Response_ProcessExpungeHandler(OnProcessExpunge);
            this.mSession.ProcessFetch += new Response_ProcessFetchHandler(OnProcessFetch);
        }

        protected void OnProcessStatus(ref ImapCommandData inCommandData, ref StatusResponse inResponse)
        {
            UInt32 responseType = inResponse.ResponseType & ~(UInt32)EnumImapStatusResponseTypes.TaggedStatusResponse;
            if ((inResponse.ResponseType & (UInt32)EnumImapStatusResponseTypes.TaggedStatusResponse) != 0)
            {
                if (inResponse.ResponseType != (UInt32)EnumImapStatusResponseTypes.TaggedOkResponse)
                    Console.Out.WriteLine("!!!Command failed: " + inResponse.StatusResponseText.Text);
            }

            ImapFolder folder = null;
            if (inCommandData.SourceMailboxName != null)
                folder = FindFolderByName(inCommandData.SourceMailboxName);

            switch (responseType)
            {
                case (UInt32)EnumImapStatusResponseTypes.OkResponse:
                {
                    switch (inResponse.StatusResponseText.Code)
                    {
                        case (UInt32)EnumImapStatusResponseCodes.PermanentFlagsCode:
                            Debug.Assert(inCommandData.SourceMailboxName != null);
                            Debug.Assert(folder != null);
                            folder.Flags = inResponse.StatusResponseText.PermanentFlags.StandardFlags;
                            StringList customFlags = inResponse.StatusResponseText.PermanentFlags.CustomFlags;
                            if (customFlags != null)
                                folder.CustomFlags = new StringList(customFlags);
                            break;
                        case (UInt32)EnumImapStatusResponseCodes.UIDNextCode:
                            Debug.Assert(inCommandData.SourceMailboxName != null);
                            Debug.Assert(folder != null);
                            folder.NextUid = inResponse.StatusResponseText.Number;
                            break;
                        case (UInt32)EnumImapStatusResponseCodes.UIDValidityCode:
                            Debug.Assert(inCommandData.SourceMailboxName != null);
                            Debug.Assert(folder != null);
                            folder.UidValidity = inResponse.StatusResponseText.Number;
                            break;
                        case (UInt32)EnumImapStatusResponseCodes.UnseenCode:
                            Debug.Assert(inCommandData.SourceMailboxName != null);
                            Debug.Assert(folder != null);
                            folder.Unseen = inResponse.StatusResponseText.Number;
                            break;
                        case (UInt32)EnumImapStatusResponseCodes.ReadOnlyCode:
                            Debug.Assert(inCommandData.SourceMailboxName != null);
                            Debug.Assert(folder != null);
                            folder.ReadOnly = true;
                            break;
                        case (UInt32)EnumImapStatusResponseCodes.ReadWriteCode:
                            Debug.Assert(inCommandData.SourceMailboxName != null);
                            Debug.Assert(folder != null);
                            folder.ReadOnly = false;
                            break;
                        case (UInt32)EnumImapStatusResponseCodes.AppendUIDCode:
                            Debug.Assert(inCommandData.DestMailboxName != null);
                            Debug.Assert(inResponse.StatusResponseText.TextUidPlus.SourceSequenceSet == null);
                            Debug.Assert(inResponse.StatusResponseText.TextUidPlus.DestSequenceSet != null);
                            folder = FindFolderByName(inCommandData.DestMailboxName);
                            //assume destination mailbox was listed previously
                            Debug.Assert(folder != null);
                            ImapSequenceSet destSequenceSe = inResponse.StatusResponseText.TextUidPlus.DestSequenceSet;
                            ImapSequenceSet sequence = new ImapSequenceSet(ref destSequenceSe);
                            folder.MessagesAppended(inCommandData.AppendedMessages, ref sequence);
                            break;
                        case (UInt32)EnumImapStatusResponseCodes.CopyUIDCode:
                            Debug.Assert(inCommandData.SourceMailboxName != null);
                            Debug.Assert(inCommandData.DestMailboxName != null);
                            Debug.Assert(inResponse.StatusResponseText.TextUidPlus.SourceSequenceSet != null);
                            Debug.Assert(inResponse.StatusResponseText.TextUidPlus.DestSequenceSet != null);
                            Debug.Assert(folder != null);
                            Console.Out.WriteLine("Not implemented yet");
                            break;
                        case (UInt32)EnumImapStatusResponseCodes.NoCode:
                            break;
                        case (UInt32)EnumImapStatusResponseCodes.XUserDefinedCode:
                            Console.Out.Write("Unrecognized status response: ");
                            if (inResponse.StatusResponseText.CodeText != null)
                                Console.Out.Write(inResponse.StatusResponseText.CodeText + " ");
                            if (inResponse.StatusResponseText.Text != null)
                                Console.Out.Write(inResponse.StatusResponseText.Text);
                            Console.Out.WriteLine();
                            break;
                        default:
                            Console.Out.WriteLine("Unhandled status response: " + inResponse.StatusResponseText.Code);
                            break;
                    }
                    break;
                }
                case (UInt32)EnumImapStatusResponseTypes.ByeResponse:
                    Console.Out.WriteLine("Disconnected");
                    break;
                case (UInt32)EnumImapStatusResponseTypes.BadResponse:
                case (UInt32)EnumImapStatusResponseTypes.NoResponse:
                    if ((inResponse.ResponseType & (UInt32)EnumImapStatusResponseTypes.TaggedStatusResponse) != 0)
                        Console.Out.WriteLine("Error: command failed with response: " +
                                    inResponse.StatusResponseText.Text);
                    else
                        Console.Out.WriteLine("Warning: command failed with response: " +
                                    inResponse.StatusResponseText.Text);
                    break;
                default:
                    Console.Out.WriteLine("Unhandled status response: " +
                                inResponse.StatusResponseText.Code);
                    break;
            }
        }

        protected void OnProcessCapability(ref CapabilityResponse inResponse)
        {
        }

        protected void OnProcessList(ref ListResponse inResponse)
        {
            ImapFolder folder = FindFolderByName(inResponse.MailboxName);
            if (folder == null)
            {
                folder = new ImapFolder(ref inResponse);
                this.mFolders.Add(folder);
            }
            else
                folder.Attributes = inResponse.Attributes;
            StringList customAttributes = inResponse.CustomAttributes;
            if (customAttributes != null)
                folder.CustomAttributes = new StringList(customAttributes);
        }

        protected void OnProcessLsub(ref LsubResponse inResponse)
        {
            ImapFolder folder = new ImapFolder(ref inResponse);
            this.mSubscribedFolders.Add(folder);
        }

        protected void OnProcessMailboxStatus(ref MailboxStatusResponse inResponse)
        {
            ImapFolder folder = FindFolderByName(inResponse.MailboxName);
            Debug.Assert(folder != null);
            folder.Exists = inResponse.Messages;
            folder.Recent = inResponse.Recent;
            folder.NextUid = inResponse.UidNext;
            folder.UidValidity = inResponse.UidValidity;
            folder.Unseen = inResponse.Unseen;
        }

        protected void OnProcessSearch(String inFolderName, ref SearchResponse inResponse)
        {
            Debug.Assert(inFolderName != null);
            ImapFolder folder = FindFolderByName(inFolderName);
            Debug.Assert(folder != null);
            ImapSequenceSet messages = inResponse.Messages;
            folder.FoundMessageList = new ImapSequenceSet(ref messages);
        }

        protected void OnProcessFlags(String inFolderName, ref FlagsResponse inResponse)
        {
            Debug.Assert(inFolderName != null);
            ImapFolder folder = FindFolderByName(inFolderName);
            Debug.Assert(folder != null);

            folder.Flags = inResponse.StandardFlags;
            StringList customFlags = inResponse.CustomFlags;
            if (customFlags != null)
                folder.CustomFlags = new StringList(customFlags);
        }

        protected void OnProcessExists(String inFolderName, ref ExistsResponse inResponse)
        {
            Debug.Assert(inFolderName != null);
            ImapFolder folder = FindFolderByName(inFolderName);
            Debug.Assert(folder != null);

            folder.Exists = inResponse.Number;
        }

        protected void OnProcessRecent(String inFolderName, ref RecentResponse inResponse)
        {
            Debug.Assert(inFolderName != null);
            ImapFolder folder = FindFolderByName(inFolderName);
            Debug.Assert(folder != null);

            folder.Recent = inResponse.Number;
        }

        protected void OnProcessExpunge(String inFolderName, ref ExpungeResponse inResponse)
        {
            Debug.Assert(inFolderName != null);
            ImapFolder folder = FindFolderByName(inFolderName);
            Debug.Assert(folder != null);

            folder.MessageDeleted(inResponse.Number);
        }

        protected void OnProcessFetch(String inFolderName, ref FetchResponse inResponse)
        {
            Debug.Assert(inFolderName != null);
            ImapFolder folder = FindFolderByName(inFolderName);
            Debug.Assert(folder != null);
            folder.AddMessageData(ref inResponse);
        }

        private ImapFolder FindFolder(ref ArrayList inList, String inName)
        {
            if (inList != null)
            {
                foreach (ImapFolder folder in inList)
                {
                    if (folder.Name.CompareTo(inName) == 0)
                        return folder;
                }
            }
            return null;
        }

        public ImapFolder FindFolderByName(String inName)
        {
            return FindFolder(ref this.mFolders, inName);
        }

        public ImapFolder FindSubscribedFolderByName(String inName)
        {
            return FindFolder(ref this.mSubscribedFolders, inName);
        }

        public void DeleteFolderFromFolderList(String inName)
        {
            for(Int32 i = 0; i < this.mFolders.Count; i++)
            {
                if (((ImapFolder)this.mFolders[i]).Name.CompareTo(inName) == 0)
                { 
                    this.mFolders.RemoveAt(i);
                }
            }
        }

        public void RenameFoldersInFolderList(String inName, String inNewName)
        {
            if (inName.CompareTo("inbox") == 0)
            {
                //Renaming INBOX is permitted, and has special behavior.  It moves 
                //all messages in INBOX to a new mailbox with the given name, 
                //leaving INBOX empty.  If the server implementation supports 
                //inferior hierarchical names of INBOX, these are unaffected by a 
                //rename of INBOX. 
                ImapFolder folder = FindFolderByName(inName);
                if (folder != null)
                    folder.Name = inNewName;

                //TODO: add object for mailbox
            }
            else
            {
                //If the name has inferior hierarchical names, then the inferior 
                //hierarchical names MUST also be renamed.  For example, a rename of 
                //"foo" to "zap" will rename "foo/bar" (assuming "/" is the 
                //hierarchy delimiter character) to "zap/bar".

                for (Int32 i = 0; i < this.mFolders.Count; i++)
                {
                    String name = inName;
                    name += ((ImapFolder)this.mFolders[i]).Delimiter;

                    String folderName = ((ImapFolder)this.mFolders[i]).Name;
                    if (folderName.CompareTo(inName) == 0)
                        ((ImapFolder)this.mFolders[i]).Name = inNewName;
                    else if (folderName.Contains(name))
                    {
                        String newName = folderName;
                        newName = inNewName + newName.Substring(name.Length - 1);
                        ((ImapFolder)this.mFolders[i]).Name = newName;
                    }
                }
            }
        }

        public Int32 FoldersCount
        {
            get
            {
                return this.mFolders.Count;
            }
        }

        public ImapFolder GetFolderAtIndex(Int32 inIndex)
        {
            Debug.Assert(inIndex <= FoldersCount - 1);
            return (ImapFolder)this.mFolders[inIndex];
        }

        public Int32 GetFolderIndex(String inName)
        {
            if (this.mFolders != null)
            {
                for (Int32 i = 0; i < this.mFolders.Count; i++)
                {
                    if (((ImapFolder)this.mFolders[i]).Name.CompareTo(inName) == 0)
                        return i;
                }
            }
            return -1;
        }

        public void ClearFolderList()
        {
            if (this.mFolders.Count > 0)
                this.mFolders.Clear();
        }

        public void ClearSubscribedFolderList()
        {
            if (this.mSubscribedFolders.Count > 0)
                this.mSubscribedFolders.Clear();
        }

    }
}
