/*
// File:		MainForm.cs
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2008 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Emai;
using Emai.Mime;
using Emai.Smtp;

namespace Smtp
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();
			this.comboBoxAuthType.SelectedIndex = 0;
		}

		private void buttonAdd_Click(object sender, EventArgs e)
		{
			if (this.openFileDialog.ShowDialog() == DialogResult.OK)
			{
				foreach (String fileName in this.openFileDialog.FileNames)
				{
					this.listViewAttachments.Items.Add("").SubItems.Add(fileName);
				}
			}
		}

		private void buttonRemove_Click(object sender, EventArgs e)
		{
			foreach (ListViewItem item in this.listViewAttachments.SelectedItems)
			{
				this.listViewAttachments.Items.RemoveAt(item.Index);
			}
		}

		private void buttonClose_Click(object sender, EventArgs e)
		{
			Application.Exit();
		}

		private void buttonSend_Click(object sender, EventArgs e)
		{
			try
			{
				Emai.License.Key = this.textBoxLicKey.Text;
				Common.MessageComposer messageComposer = new Common.MessageComposer();
				this.toolStripStatusLabelText.Text = "Composing message...";
				if (this.checkBoxAttachPlainText.Checked)
				{
					messageComposer.UnicodeText = this.textBoxPlainText.Text;
				}

				String resHtml = this.textBoxHtmlText.Text;
				foreach (ListViewItem item in this.listViewAttachments.Items)
				{
					String name = Path.GetFileName(item.SubItems[1].Text);
					if (item.Checked)
					{
						if (this.checkBoxAttachHtmlText.Checked)
						{
							int index = resHtml.IndexOf("</body>");
							String left = resHtml.Substring(0, index);
							String right = resHtml.Substring(index);
							String html = left;
							String tag = String.Format("<img src=\"cid:{0:S}\">", name);
							if (resHtml.IndexOf(tag) < 0)
							{
								html += tag;
							}
							html += right;
							resHtml = html;
						}
						messageComposer.AddInlineFromFile(name, item.SubItems[1].Text, name);
					}
					else
					{
						messageComposer.AddAttachmentFromFile(name, item.SubItems[1].Text);
					}
				}

				if (this.checkBoxAttachHtmlText.Checked)
					messageComposer.Html = resHtml;

				ContactList contactTo = new ContactList();
				contactTo.AddContact(this.textBoxTo.Text, null, (UInt32)EnumNullOptions.NullOptions);
				ContactList contactFrom = new ContactList();
				contactFrom.AddContact(this.textBoxFrom.Text, null, (UInt32)EnumNullOptions.NullOptions);

				Emai.Mime.Message message = messageComposer.ComposeMessage(contactFrom, null, contactTo,
					null, null, this.textBoxSubject.Text);
				this.toolStripStatusLabelText.Text = "Message successfully composed.";

				if (message != null)
				{
					UInt32 options = (UInt32)EnumNullOptions.NullOptions;
					if (this.checkBoxUseSsl.Checked)
						options = (UInt32)EnumSmtpSSL.SmtpUseSSLOption;
					SmtpSession smtpSession = new SmtpSession(60, options);

					smtpSession.OnOperation += new Event_OnOperationEventHandler(this.mailit_OnOperation);
					smtpSession.OnDataSent += new Event_OnDataEventHandler(this.mailit_OnDataSent);
					smtpSession.OnServerError += new Event_OnServerErrorEventHandler(this.mailit_OnServerError);

					smtpSession.Connect(this.textBoxServer.Text, (short)this.numericUpDownPort.Value, (UInt32)EnumNullOptions.NullOptions);

					UInt32[] authTypes = 
                {
		            (UInt32)EnumSmtpAuthSelector.SMTPAuthAutomatic,
		            (UInt32)EnumSmtpAuthSelector.SMTPAuthLogin,
		            (UInt32)EnumSmtpAuthSelector.SMTPAuthPlain,
		            (UInt32)EnumSmtpAuthSelector.SMTPAuthCramMD5,
		            (UInt32)EnumSmtpAuthSelector.SMTPAuthNTLM 
                };

					smtpSession.Authenticate(this.textBoxAccount.Text, this.textBoxPassword.Text, authTypes[this.comboBoxAuthType.SelectedIndex]);
					/*Send the message*/
					smtpSession.SendMessage(message, (UInt32)EnumNullOptions.NullOptions);

					/*Disconnect from the server*/
					smtpSession.Disconnect();
				}
			}
			catch (InvalidLicenseException exception)
			{
				MessageBox.Show("Licence key is invalid. Restart application and enter valid key.", "Error");
				this.toolStripStatusLabelText.Text = "";
				Console.Out.WriteLine(exception.ErrorCode);
			}
			catch (Emai.Exception exception)
			{
				this.toolStripStatusLabelText.Text = "";
				Console.Out.WriteLine(exception.ErrorCode);
			}

			this.toolStripStatusLabelText.Text = "Ready";
		}

		private void checkBoxAttachPlainText_CheckedChanged(object sender, EventArgs e)
		{
			this.textBoxPlainText.ReadOnly = !this.checkBoxAttachPlainText.Checked;
		}

		private void checkBoxAttachHtmlText_CheckedChanged(object sender, EventArgs e)
		{
			this.textBoxHtmlText.ReadOnly = !this.checkBoxAttachHtmlText.Checked;
		}

		private void checkBoxUseSsl_CheckedChanged(object sender, EventArgs e)
		{
			this.toolStripStatusLabelText.Text = "Connection successfully closed.";
		}

		private void listViewAttachments_Resize(object sender, EventArgs e)
		{
			this.listViewAttachments.Columns[0].Width = (this.listViewAttachments.ClientSize.Width * 10) / 100;
			this.listViewAttachments.Columns[1].Width = (this.listViewAttachments.ClientSize.Width * 90) / 100;
		}

		private void listViewAttachments_ItemActivate(object sender, EventArgs e)
		{
			if (this.checkBoxAttachHtmlText.Checked)
			{
				foreach (ListViewItem item in this.listViewAttachments.SelectedItems)
				{
					this.textBoxHtmlText.Text = this.textBoxHtmlText.Text.Insert(this.textBoxHtmlText.SelectionStart,
						String.Format("<img src=\"cid:{0:S}\">", Path.GetFileName(item.SubItems[1].Text)));
				}
			}
			else
				MessageBox.Show("You can not add inline attachment without html message part.");
		}

		private void mailit_OnDataSent(object sender, DataEventArgs e)
		{
			Int32 curPercent = (100 * (Int32)e.TransferedSize) / (Int32)e.TotalSize;
			this.toolStripProgressBarPercent.Value = curPercent;
		}

		private void mailit_OnServerError(object sender, ServerErrorEventArgs e)
		{
			this.toolStripStatusLabelText.Text = "The server error encountered was: " + e.ServerReply;
		}

		private void mailit_OnOperation(object sender, OperationEventArgs e)
		{
			String[] statusTexts = {
            "Connecting to server...",
			"Successfully connected to server.",
			"Authenticating on server...",
			"Successfully authenticated on server.",
			"Introducing sender...",
			"Successfully introduced sender.",
			"Sending recipients...",
			"Recipients successfully sent.",
			"Sending message data...",
			"Message data successfully sent.",
			"Sending NOOP...",
			"NOOP successfully sent.",
			"Closing connection.",
			"Connection successfully closed."
            };
			this.toolStripStatusLabelText.Text = statusTexts[e.OperationCode];
			this.Refresh();
		}

		private void textBoxLicKey_Enter(object sender, EventArgs e)
		{
			if (this.textBoxLicKey.Text.CompareTo("Put your license key here.") == 0)
			{
				this.textBoxLicKey.Text = "";
			}
		}

		private void textBoxLicKey_Leave(object sender, EventArgs e)
		{
			if (this.textBoxLicKey.Text.Length == 0)
			{
				this.textBoxLicKey.Text = "Put your license key here.";
			}
		}
	}
}